@@ -2,7 +2,7 @@
 #![cfg_attr(test, deny(warnings))]
 #![deny(missing_docs)]
 
-//! Wrappers for total order on Floats.
+//! Wrappers for total order on Floats.  See the [`OrderedFloat`] and [`NotNan`] docs for details.
 
 #[cfg(feature = "std")] extern crate std;
 #[cfg(feature = "std")] use std::error::Error;
@@ -34,10 +34,32 @@ const MAN_MASK: u64 = 0x000fffffffffffffu64;
 const CANONICAL_NAN_BITS: u64 = 0x7ff8000000000000u64;
 const CANONICAL_ZERO_BITS: u64 = 0x0u64;
 
-/// A wrapper around Floats providing an implementation of Ord and Hash.
+/// A wrapper around floats providing implementations of `Eq`, `Ord`, and `Hash`.
 ///
 /// NaN is sorted as *greater* than all other values and *equal*
 /// to itself, in contradiction with the IEEE standard.
+///
+/// ```
+/// use ordered_float::OrderedFloat;
+/// use std::f32::NAN;
+///
+/// let mut v = [OrderedFloat(NAN), OrderedFloat(2.0), OrderedFloat(1.0)];
+/// v.sort();
+/// assert_eq!(v, [OrderedFloat(1.0), OrderedFloat(2.0), OrderedFloat(NAN)]);
+/// ```
+///
+/// Because `OrderedFloat` implements `Ord` and `Eq`, it can be used as a key in a `HashSet`,
+/// `HashMap`, `BTreeMap`, or `BTreeSet` (unlike the primitive `f32` or `f64` types):
+///
+/// ```
+/// # use ordered_float::OrderedFloat;
+/// # use std::collections::HashSet;
+/// # use std::f32::NAN;
+///
+/// let mut s: HashSet<OrderedFloat<f32>> = HashSet::new();
+/// s.insert(OrderedFloat(NAN));
+/// assert!(s.contains(&OrderedFloat(NAN)));
+/// ```
 #[derive(Debug, Default, Clone, Copy)]
 #[repr(transparent)]
 pub struct OrderedFloat<T>(pub T);
@@ -173,85 +195,96 @@ impl<T: Float> DerefMut for OrderedFloat<T> {
 
 impl<T: Float> Eq for OrderedFloat<T> {}
 
-impl<T: Add> Add for OrderedFloat<T> {
-    type Output = OrderedFloat<T::Output>;
+macro_rules! impl_ordered_float_binop {
+    ($imp:ident, $method:ident) => {
+        impl<T: $imp> $imp for OrderedFloat<T> {
+            type Output = OrderedFloat<T::Output>;
 
-    fn add(self, other: Self) -> Self::Output {
-        OrderedFloat(self.0 + other.0)
+            #[inline]
+            fn $method(self, other: Self) -> Self::Output {
+                OrderedFloat((self.0).$method(other.0))
+            }
+        }
+
+        impl<T: $imp> $imp<T> for OrderedFloat<T> {
+            type Output = OrderedFloat<T::Output>;
+
+            #[inline]
+            fn $method(self, other: T) -> Self::Output {
+                OrderedFloat((self.0).$method(other))
+            }
+        }
+
+        impl<'a, T> $imp<&'a T> for OrderedFloat<T> where T: $imp<&'a T> {
+            type Output = OrderedFloat<<T as $imp<&'a T>>::Output>;
+
+            #[inline]
+            fn $method(self, other: &'a T) -> Self::Output {
+                OrderedFloat((self.0).$method(other))
+            }
+        }
+
+        impl<'a, T> $imp<&'a Self> for OrderedFloat<T> where T: $imp<&'a T> {
+            type Output = OrderedFloat<<T as $imp<&'a T>>::Output>;
+
+            #[inline]
+            fn $method(self, other: &'a Self) -> Self::Output {
+                OrderedFloat((self.0).$method(&other.0))
+            }
+        }
+
+        impl<'a, T> $imp for &'a OrderedFloat<T> where &'a T: $imp {
+            type Output = OrderedFloat<<&'a T as $imp>::Output>;
+
+            #[inline]
+            fn $method(self, other: Self) -> Self::Output {
+                OrderedFloat((self.0).$method(&other.0))
+            }
+        }
+
+        impl<'a, T> $imp<OrderedFloat<T>> for &'a OrderedFloat<T> where &'a T: $imp<T> {
+            type Output = OrderedFloat<<&'a T as $imp<T>>::Output>;
+
+            #[inline]
+            fn $method(self, other: OrderedFloat<T>) -> Self::Output {
+                OrderedFloat((self.0).$method(other.0))
+            }
+        }
+
+        impl<'a, T> $imp<T> for &'a OrderedFloat<T> where &'a T: $imp<T> {
+            type Output = OrderedFloat<<&'a T as $imp<T>>::Output>;
+
+            #[inline]
+            fn $method(self, other: T) -> Self::Output {
+                OrderedFloat((self.0).$method(other))
+            }
+        }
+
+        impl<'a, T> $imp<&'a T> for &'a OrderedFloat<T> where &'a T: $imp {
+            type Output = OrderedFloat<<&'a T as $imp>::Output>;
+
+            #[inline]
+            fn $method(self, other: &'a T) -> Self::Output {
+                OrderedFloat((self.0).$method(other))
+            }
+        }
+
+        impl<'a, T> $imp<&'a Self> for &'a OrderedFloat<T> where &'a T: $imp {
+            type Output = OrderedFloat<<&'a T as $imp>::Output>;
+
+            #[inline]
+            fn $method(self, other: &'a Self) -> Self::Output {
+                OrderedFloat((self.0).$method(&other.0))
+            }
+        }
     }
 }
 
-impl<T: Add> Add<T> for OrderedFloat<T> {
-    type Output = OrderedFloat<T::Output>;
-
-    fn add(self, other: T) -> Self::Output {
-        OrderedFloat(self.0 + other)
-    }
-}
-
-impl<T: Sub> Sub for OrderedFloat<T> {
-    type Output = OrderedFloat<T::Output>;
-
-    fn sub(self, other: Self) -> Self::Output {
-        OrderedFloat(self.0 - other.0)
-    }
-}
-
-impl<T: Sub> Sub<T> for OrderedFloat<T> {
-    type Output = OrderedFloat<T::Output>;
-
-    fn sub(self, other: T) -> Self::Output {
-        OrderedFloat(self.0 - other)
-    }
-}
-
-impl<T: Mul> Mul for OrderedFloat<T> {
-    type Output = OrderedFloat<T::Output>;
-
-    fn mul(self, other: Self) -> Self::Output {
-        OrderedFloat(self.0 * other.0)
-    }
-}
-
-impl<T: Mul> Mul<T> for OrderedFloat<T> {
-    type Output = OrderedFloat<T::Output>;
-
-    fn mul(self, other: T) -> Self::Output {
-        OrderedFloat(self.0 * other)
-    }
-}
-
-impl<T: Div> Div for OrderedFloat<T> {
-    type Output = OrderedFloat<T::Output>;
-
-    fn div(self, other: Self) -> Self::Output {
-        OrderedFloat(self.0 / other.0)
-    }
-}
-
-impl<T: Div> Div<T> for OrderedFloat<T> {
-    type Output = OrderedFloat<T::Output>;
-
-    fn div(self, other: T) -> Self::Output {
-        OrderedFloat(self.0 / other)
-    }
-}
-
-impl<T: Rem> Rem for OrderedFloat<T> {
-    type Output = OrderedFloat<T::Output>;
-
-    fn rem(self, other: Self) -> Self::Output {
-        OrderedFloat(self.0 % other.0)
-    }
-}
-
-impl<T: Rem> Rem<T> for OrderedFloat<T> {
-    type Output = OrderedFloat<T::Output>;
-
-    fn rem(self, other: T) -> Self::Output {
-        OrderedFloat(self.0 % other)
-    }
-}
+impl_ordered_float_binop!{Add, add}
+impl_ordered_float_binop!{Sub, sub}
+impl_ordered_float_binop!{Mul, mul}
+impl_ordered_float_binop!{Div, div}
+impl_ordered_float_binop!{Rem, rem}
 
 impl<T: Bounded> Bounded for OrderedFloat<T> {
     fn min_value() -> Self {
@@ -288,6 +321,14 @@ impl<T: Neg> Neg for OrderedFloat<T> {
     }
 }
 
+impl<'a, T> Neg for &'a OrderedFloat<T> where &'a T: Neg {
+    type Output = OrderedFloat<<&'a T as Neg>::Output>;
+
+    fn neg(self) -> Self::Output {
+        OrderedFloat(-(&self.0))
+    }
+}
+
 impl<T: Zero> Zero for OrderedFloat<T> {
     fn zero() -> Self { OrderedFloat(T::zero()) }
 
@@ -436,9 +477,44 @@ impl<T: Float + Num> Num for OrderedFloat<T> {
     }
 }
 
-/// A wrapper around Floats providing an implementation of Ord and Hash.
+/// A wrapper around floats providing an implementation of `Eq`, `Ord` and `Hash`.
 ///
 /// A NaN value cannot be stored in this type.
+///
+/// ```
+/// use ordered_float::NotNan;
+///
+/// let mut v = [
+///     NotNan::new(2.0).unwrap(), 
+///     NotNan::new(1.0).unwrap(), 
+/// ];
+/// v.sort();
+/// assert_eq!(v, [1.0, 2.0]);
+/// ```
+///
+/// Because `NotNan` implements `Ord` and `Eq`, it can be used as a key in a `HashSet`,
+/// `HashMap`, `BTreeMap`, or `BTreeSet` (unlike the primitive `f32` or `f64` types):
+///
+/// ```
+/// # use ordered_float::NotNan;
+/// # use std::collections::HashSet;
+///
+/// let mut s: HashSet<NotNan<f32>> = HashSet::new();
+/// let key = NotNan::new(1.0).unwrap();
+/// s.insert(key);
+/// assert!(s.contains(&key));
+/// ```
+///
+/// Arithmetic on NotNan values will panic if it produces a NaN value:
+///
+/// ```should_panic
+/// # use ordered_float::NotNan;
+/// let a = NotNan::new(std::f32::INFINITY).unwrap();
+/// let b = NotNan::new(std::f32::NEG_INFINITY).unwrap();
+///
+/// // This will panic:
+/// let c = a + b;
+/// ```
 #[derive(PartialOrd, PartialEq, Debug, Default, Clone, Copy)]
 #[repr(transparent)]
 pub struct NotNan<T>(T);
@@ -549,17 +625,6 @@ impl<T: Float> PartialEq<T> for NotNan<T> {
     }
 }
 
-/// Adds two NotNans.
-///
-/// Panics if the computation results in NaN
-impl<T: Float> Add for NotNan<T> {
-    type Output = Self;
-
-    fn add(self, other: Self) -> Self {
-        self + other.0
-    }
-}
-
 /// Adds a float directly.
 ///
 /// Panics if the provided value is NaN or the computation results in NaN
@@ -571,22 +636,9 @@ impl<T: Float> Add<T> for NotNan<T> {
     }
 }
 
-impl<T: Float + AddAssign> AddAssign for NotNan<T> {
-    fn add_assign(&mut self, other: Self) {
-        *self += other.0;
-    }
-}
-
 /// Adds a float directly.
 ///
 /// Panics if the provided value is NaN.
-impl<T: Float + AddAssign> AddAssign<T> for NotNan<T> {
-    fn add_assign(&mut self, other: T) {
-        *self = *self + other;
-    }
-}
-
-
 impl<T: Float + Sum> Sum for NotNan<T> {
     fn sum<I: Iterator<Item = NotNan<T>>>(iter: I) -> Self {
         NotNan::new(iter.map(|v| v.0).sum()).expect("Sum resulted in NaN")
@@ -599,14 +651,6 @@ impl<'a, T: Float + Sum + 'a> Sum<&'a NotNan<T>> for NotNan<T> {
     }
 }
 
-impl<T: Float> Sub for NotNan<T> {
-    type Output = Self;
-
-    fn sub(self, other: Self) -> Self {
-        self - other.0
-    }
-}
-
 /// Subtracts a float directly.
 ///
 /// Panics if the provided value is NaN or the computation results in NaN
@@ -618,29 +662,6 @@ impl<T: Float> Sub<T> for NotNan<T> {
     }
 }
 
-impl<T: Float + SubAssign> SubAssign for NotNan<T> {
-    fn sub_assign(&mut self, other: Self) {
-        *self -= other.0
-    }
-}
-
-/// Subtracts a float directly.
-///
-/// Panics if the provided value is NaN or the computation results in NaN
-impl<T: Float + SubAssign> SubAssign<T> for NotNan<T> {
-    fn sub_assign(&mut self, other: T) {
-        *self = *self - other;
-    }
-}
-
-impl<T: Float> Mul for NotNan<T> {
-    type Output = Self;
-
-    fn mul(self, other: Self) -> Self {
-        self * other.0
-    }
-}
-
 /// Multiplies a float directly.
 ///
 /// Panics if the provided value is NaN or the computation results in NaN
@@ -652,21 +673,6 @@ impl<T: Float> Mul<T> for NotNan<T> {
     }
 }
 
-impl<T: Float + MulAssign> MulAssign for NotNan<T> {
-    fn mul_assign(&mut self, other: Self) {
-        *self *= other.0
-    }
-}
-
-/// Multiplies a float directly.
-///
-/// Panics if the provided value is NaN.
-impl<T: Float + MulAssign> MulAssign<T> for NotNan<T> {
-    fn mul_assign(&mut self, other: T) {
-        *self = *self * other;
-    }
-}
-
 impl<T: Float + Product> Product for NotNan<T> {
     fn product<I: Iterator<Item = NotNan<T>>>(iter: I) -> Self {
         NotNan::new(iter.map(|v| v.0).product()).expect("Product resulted in NaN")
@@ -679,14 +685,6 @@ impl<'a, T: Float + Product + 'a> Product<&'a NotNan<T>> for NotNan<T> {
     }
 }
 
-impl<T: Float> Div for NotNan<T> {
-    type Output = Self;
-
-    fn div(self, other: Self) -> Self {
-        self / other.0
-    }
-}
-
 /// Divides a float directly.
 ///
 /// Panics if the provided value is NaN or the computation results in NaN
@@ -698,29 +696,6 @@ impl<T: Float> Div<T> for NotNan<T> {
     }
 }
 
-impl<T: Float + DivAssign> DivAssign for NotNan<T> {
-    fn div_assign(&mut self, other: Self) {
-        *self /= other.0;
-    }
-}
-
-/// Divides a float directly.
-///
-/// Panics if the provided value is NaN or the computation results in NaN
-impl<T: Float + DivAssign> DivAssign<T> for NotNan<T> {
-    fn div_assign(&mut self, other: T) {
-        *self = *self / other;
-    }
-}
-
-impl<T: Float> Rem for NotNan<T> {
-    type Output = Self;
-
-    fn rem(self, other: Self) -> Self {
-        self % other.0
-    }
-}
-
 /// Calculates `%` with a float directly.
 ///
 /// Panics if the provided value is NaN or the computation results in NaN
@@ -732,20 +707,95 @@ impl<T: Float> Rem<T> for NotNan<T> {
     }
 }
 
-impl<T: Float + RemAssign> RemAssign for NotNan<T> {
-    fn rem_assign(&mut self, other: Self) {
-        *self %= other.0
+macro_rules! impl_not_nan_binop {
+    ($imp:ident, $method:ident, $assign_imp:ident, $assign_method:ident) => {
+        impl<T: Float> $imp for NotNan<T> {
+            type Output = Self;
+
+            fn $method(self, other: Self) -> Self {
+                self.$method(other.0)
+            }
+        }
+
+        impl<T: Float + Copy> $imp<&T> for NotNan<T> {
+            type Output = NotNan<T>;
+
+            fn $method(self, other: &T) -> Self::Output {
+                self.$method(*other)
+            }
+        }
+
+        impl<T: Float + Copy> $imp<&Self> for NotNan<T> {
+            type Output = NotNan<T>;
+
+            fn $method(self, other: &Self) -> Self::Output {
+                self.$method(other.0)
+            }
+        }
+
+        impl<T: Float + Copy> $imp for &NotNan<T> {
+            type Output = NotNan<T>;
+
+            fn $method(self, other: Self) -> Self::Output {
+                (*self).$method(other.0)
+            }
+        }
+
+        impl<T: Float + Copy> $imp<NotNan<T>> for &NotNan<T> {
+            type Output = NotNan<T>;
+
+            fn $method(self, other: NotNan<T>) -> Self::Output {
+                (*self).$method(other.0)
+            }
+        }
+
+        impl<T: Float + Copy> $imp<T> for &NotNan<T> {
+            type Output = NotNan<T>;
+
+            fn $method(self, other: T) -> Self::Output {
+                (*self).$method(other)
+            }
+        }
+
+        impl<T: Float + Copy> $imp<&T> for &NotNan<T> {
+            type Output = NotNan<T>;
+
+            fn $method(self, other: &T) -> Self::Output {
+                (*self).$method(*other)
+            }
+        }
+
+        impl<T: Float + $assign_imp> $assign_imp<T> for NotNan<T> {
+            fn $assign_method(&mut self, other: T) {
+                *self = (*self).$method(other);
+            }
+        }
+
+        impl<T: Float + $assign_imp> $assign_imp<&T> for NotNan<T> {
+            fn $assign_method(&mut self, other: &T) {
+                *self = (*self).$method(*other);
+            }
+        }
+
+        impl<T: Float + $assign_imp> $assign_imp for NotNan<T> {
+            fn $assign_method(&mut self, other: Self) {
+                (*self).$assign_method(other.0);
+            }
+        }
+
+        impl<T: Float + $assign_imp> $assign_imp<&Self> for NotNan<T> {
+            fn $assign_method(&mut self, other: &Self) {
+                (*self).$assign_method(other.0);
+            }
+        }
     }
 }
 
-/// Calculates `%=` with a float directly.
-///
-/// Panics if the provided value is NaN or the computation results in NaN
-impl<T: Float + RemAssign> RemAssign<T> for NotNan<T> {
-    fn rem_assign(&mut self, other: T) {
-        *self = *self % other;
-    }
-}
+impl_not_nan_binop!{Add, add, AddAssign, add_assign}
+impl_not_nan_binop!{Sub, sub, SubAssign, sub_assign}
+impl_not_nan_binop!{Mul, mul, MulAssign, mul_assign}
+impl_not_nan_binop!{Div, div, DivAssign, div_assign}
+impl_not_nan_binop!{Rem, rem, RemAssign, rem_assign}
 
 impl<T: Float> Neg for NotNan<T> {
     type Output = Self;
@@ -755,6 +805,14 @@ impl<T: Float> Neg for NotNan<T> {
     }
 }
 
+impl<T: Float> Neg for &NotNan<T> {
+    type Output = NotNan<T>;
+
+    fn neg(self) -> Self::Output {
+        NotNan(-self.0)
+    }
+}
+
 /// An error indicating an attempt to construct NotNan from a NaN
 #[derive(Copy, Clone, PartialEq, Eq, Debug)]
 pub struct FloatIsNan;
